@@ -130,12 +130,16 @@ class ChatControlBase(MessageControl):
 		self.handlers[id] = self.widget
 
 		widget = self.xml.get_widget('banner_eventbox')
+		widget.set_property('height-request', gajim.config.get('chat_avatar_height'))
 		id = widget.connect('button-press-event',
 			self._on_banner_eventbox_button_press_event)
 		self.handlers[id] = widget
 
 		# Create textviews and connect signals
 		self.conv_textview = ConversationTextview(self.account)
+		id = self.conv_textview.tv.connect('key_press_event',
+			self._conv_textview_key_press_event)
+		self.handlers[id] = self.conv_textview.tv
 
 		self.conv_scrolledwindow = self.xml.get_widget(
 			'conversation_scrolledwindow')
@@ -144,6 +148,11 @@ class ChatControlBase(MessageControl):
 		id = widget.connect('value-changed',
 			self.on_conversation_vadjustment_value_changed)
 		self.handlers[id] = widget
+		id = widget.connect('changed',
+			self.on_conversation_vadjustment_changed)
+		self.handlers[id] = widget
+		self.scroll_to_end_id = None
+		self.was_at_the_end = True
 		# add MessageTextView to UI and connect signals
 		self.msg_scrolledwindow = self.xml.get_widget('message_scrolledwindow')
 		self.msg_textview = MessageTextView()
@@ -329,6 +338,13 @@ class ChatControlBase(MessageControl):
 			widget.modify_fg(gtk.STATE_NORMAL, fg_color)
 		self.connect_style_event(widget, opts[0], opts[1])
 	
+	def _conv_textview_key_press_event(self, widget, event):
+		if gtk.gtk_version < (2, 12, 0):
+			return
+		if event.state & (gtk.gdk.SHIFT_MASK | gtk.gdk.CONTROL_MASK):
+			return False
+		self.parent_win.notebook.emit('key_press_event', event)
+
 	def _on_keypress_event(self, widget, event):
 		if event.state & gtk.gdk.CONTROL_MASK:
 			# CTRL + l|L: clear conv_textview
@@ -504,7 +520,7 @@ class ChatControlBase(MessageControl):
 		return False
 
 	def send_message(self, message, keyID = '', type = 'chat', chatstate = None,
-	msg_id = None, composing_jep = None, resource = None):
+	msg_id = None, composing_xep = None, resource = None):
 		'''Send the given message to the active tab. Doesn't return None if error
 		'''
 		if not message or message == '\n':
@@ -514,7 +530,7 @@ class ChatControlBase(MessageControl):
 		if not self._process_command(message):
 			ret = MessageControl.send_message(self, message, keyID, type = type,
 				chatstate = chatstate, msg_id = msg_id,
-				composing_jep = composing_jep, resource = resource,
+				composing_xep = composing_xep, resource = resource,
 				user_nick = self.user_nick)
 			if ret:
 				return ret
@@ -551,7 +567,7 @@ class ChatControlBase(MessageControl):
 		full_jid = self.get_full_jid()
 		textview = self.conv_textview
 		end = False
-		if textview.at_the_end() or kind == 'outgoing':
+		if self.was_at_the_end or kind == 'outgoing':
 			end = True
 		textview.print_conversation_line(text, jid, kind, name, tim,
 			other_tags_for_name, other_tags_for_time, other_tags_for_text,
@@ -657,7 +673,7 @@ class ChatControlBase(MessageControl):
 	def set_control_active(self, state):
 		if state:
 			jid = self.contact.jid
-			if self.conv_textview.at_the_end():
+			if self.was_at_the_end:
 				# we are at the end
 				type_ = 'printed_' + self.type_id
 				if self.type_id == message_control.TYPE_GC:
@@ -672,18 +688,23 @@ class ChatControlBase(MessageControl):
 
 	def bring_scroll_to_end(self, textview, diff_y = 0):
 		''' scrolls to the end of textview if end is not visible '''
+		if self.scroll_to_end_id:
+			# a scroll is already planned
+			return
 		buffer = textview.get_buffer()
 		end_iter = buffer.get_end_iter()
 		end_rect = textview.get_iter_location(end_iter)
 		visible_rect = textview.get_visible_rect()
 		# scroll only if expected end is not visible
 		if end_rect.y >= (visible_rect.y + visible_rect.height + diff_y):
-			gobject.idle_add(self.scroll_to_end_iter, textview)
+			self.scroll_to_end_id = gobject.idle_add(self.scroll_to_end_iter,
+				textview)
 
 	def scroll_to_end_iter(self, textview):
 		buffer = textview.get_buffer()
 		end_iter = buffer.get_end_iter()
 		textview.scroll_to_iter(end_iter, 0, False, 1, 1)
+		self.scroll_to_end_id = None
 		return False
 
 	def size_request(self, msg_textview , requisition):
@@ -741,7 +762,15 @@ class ChatControlBase(MessageControl):
 
 		return True
 
-	def on_conversation_vadjustment_value_changed(self, widget):
+	def on_conversation_vadjustment_changed(self, adjustment):
+		# used to stay at the end of the textview when we shrink conversation
+		# textview.
+		if self.was_at_the_end:
+			self.conv_textview.bring_scroll_to_end(-18)
+		self.was_at_the_end = (adjustment.upper - adjustment.value - adjustment.page_size) < 18
+
+	def on_conversation_vadjustment_value_changed(self, adjustment):
+		self.was_at_the_end = (adjustment.upper - adjustment.value - adjustment.page_size) < 18
 		if self.resource:
 			jid = self.contact.get_full_jid()
 		else:
@@ -753,12 +782,9 @@ class ChatControlBase(MessageControl):
 		type_])):
 			return
 		if self.conv_textview.at_the_end() and \
-				self.parent_win.get_active_control() == self and \
-				self.parent_win.window.is_active():
+		self.parent_win.get_active_control() == self and \
+		self.parent_win.window.is_active():
 			# we are at the end
-			type_ = self.type_id
-			if type_ == message_control.TYPE_GC:
-				type_ = 'gc_msg'
 			if not gajim.events.remove_events(self.account, self.get_full_jid(),
 			types = ['printed_' + type_, type_]):
 				# There were events to remove
@@ -1027,6 +1053,8 @@ class ChatControl(ChatControlBase):
 		jid = contact.jid
 
 		banner_name_label = self.xml.get_widget('banner_name_label')
+		banner_eventbox = self.xml.get_widget('banner_eventbox')
+		
 		name = contact.get_shown_name()
 		if self.resource:
 			name += '/' + self.resource
@@ -1053,8 +1081,10 @@ class ChatControl(ChatControlBase):
 
 		status = contact.status
 		if status is not None:
+			self.status_tooltip.set_tip(banner_eventbox, status)
+			self.status_tooltip.enable()
 			banner_name_label.set_ellipsize(pango.ELLIPSIZE_END)
-			status = helpers.reduce_chars_newlines(status, max_lines = 2)
+			status = helpers.reduce_chars_newlines(status, max_lines = 1)
 		status_escaped = gtkgui_helpers.escape_for_pango_markup(status)
 
 		font_attrs, font_attrs_small = self.get_font_attrs()
@@ -1063,12 +1093,12 @@ class ChatControl(ChatControlBase):
 		if cs and st in ('composing_only', 'all'):
 			if contact.show == 'offline':
 				chatstate = ''
-			elif contact.composing_jep == 'JEP-0085':
+			elif contact.composing_xep == 'XEP-0085':
 				if st == 'all' or cs == 'composing':
 					chatstate = helpers.get_uf_chatstate(cs)
 				else:
 					chatstate = ''
-			elif contact.composing_jep == 'JEP-0022':
+			elif contact.composing_xep == 'XEP-0022':
 				if cs in ('composing', 'paused'):
 					# only print composing, paused
 					chatstate = helpers.get_uf_chatstate(cs)
@@ -1087,9 +1117,6 @@ class ChatControl(ChatControlBase):
 		if status_escaped:
 			label_text += '\n<span %s>%s</span>' %\
 				(font_attrs_small, status_escaped)
-			banner_eventbox = self.xml.get_widget('banner_eventbox')
-			self.status_tooltip.set_tip(banner_eventbox, status)
-			self.status_tooltip.enable()
 		else:
 			self.status_tooltip.disable()
 		# setup the label that holds name and jid
@@ -1143,29 +1170,25 @@ class ChatControl(ChatControlBase):
 
 		chatstates_on = gajim.config.get('outgoing_chat_state_notifications') != \
 			'disabled'
-		composing_jep = contact.composing_jep
+		composing_xep = contact.composing_xep
 		chatstate_to_send = None
 		if chatstates_on and contact is not None:
-			if composing_jep is None:
+			if composing_xep is None:
 				# no info about peer
 				# send active to discover chat state capabilities
 				# this is here (and not in send_chatstate)
 				# because we want it sent with REAL message
 				# (not standlone) eg. one that has body
-				
-				#FIXME:
-				# Enable 3 next lines after 0.11 release.
-				# Having this disabled violate xep85 5.1.2 but then we don't break
-				# notifications between 0.10.1 and 0.11 See #2637
-				# if contact.our_chatstate:
-				#	# We already ask for xep 85, don't ask it twice
-				#	composing_jep = 'asked_once'
+
+				if contact.our_chatstate:
+					# We already asked for xep 85, don't ask it twice
+					composing_xep = 'asked_once'
 
 				chatstate_to_send = 'active'
 				contact.our_chatstate = 'ask' # pseudo state
 			# if peer supports jep85 and we are not 'ask', send 'active'
 			# NOTE: first active and 'ask' is set in gajim.py
-			elif composing_jep is not False:
+			elif composing_xep is not False:
 				#send active chatstate on every message (as JEP says)
 				chatstate_to_send = 'active'
 				contact.our_chatstate = 'active'
@@ -1175,7 +1198,7 @@ class ChatControl(ChatControlBase):
 				self._schedule_activity_timers()
 				
 		if not ChatControlBase.send_message(self, message, keyID, type = 'chat',
-		chatstate = chatstate_to_send, composing_jep = composing_jep):
+		chatstate = chatstate_to_send, composing_xep = composing_xep):
 			self.print_conversation(message, self.contact.jid,
 				encrypted = encrypted)
 
@@ -1468,7 +1491,7 @@ class ChatControl(ChatControlBase):
 		if contact.show == 'offline':
 			return
 
-		if contact.composing_jep is False: # jid cannot do jep85 nor jep22
+		if contact.composing_xep is False: # jid cannot do xep85 nor xep22
 			return
 
 		# if the new state we wanna send (state) equals 
@@ -1476,7 +1499,7 @@ class ChatControl(ChatControlBase):
 		if contact.our_chatstate == state:
 			return
 
-		if contact.composing_jep is None:
+		if contact.composing_xep is None:
 			# we don't know anything about jid, so return
 			# NOTE:
 			# send 'active', set current state to 'ask' and return is done
@@ -1490,7 +1513,7 @@ class ChatControl(ChatControlBase):
 
 		# in JEP22, when we already sent stop composing
 		# notification on paused, don't resend it
-		if contact.composing_jep == 'JEP-0022' and \
+		if contact.composing_xep == 'XEP-0022' and \
 		contact.our_chatstate in ('paused', 'active', 'inactive') and \
 		state is not 'composing': # not composing == in (active, inactive, gone)
 			contact.our_chatstate = 'active'
@@ -1512,7 +1535,7 @@ class ChatControl(ChatControlBase):
 			self.reset_kbd_mouse_timeout_vars()
 
 		MessageControl.send_message(self, None, chatstate = state,
-			msg_id = contact.msg_id, composing_jep = contact.composing_jep)
+			msg_id = contact.msg_id, composing_xep = contact.composing_xep)
 		contact.our_chatstate = state
 		if contact.our_chatstate == 'active':
 			self.reset_kbd_mouse_timeout_vars()
@@ -1575,24 +1598,34 @@ class ChatControl(ChatControlBase):
 		if not gajim.config.get('show_avatar_in_chat'):
 			return
 
-		jid = self.contact.jid
-		jid_with_resource = jid
-		if resource:
-			jid_with_resource += '/' + resource
+		is_fake = False
+		if self.TYPE_ID == message_control.TYPE_PM:
+			is_fake = True
+			jid_with_resource = self.contact.jid # fake jid
+		else:
+			jid_with_resource = self.contact.jid
+			if resource:
+				jid_with_resource += '/' + resource
 
 		# we assume contact has no avatar
 		scaled_pixbuf = None
 
-		pixbuf = None
-		is_fake = False
-		if gajim.contacts.is_pm_from_jid(self.account, jid):
-			is_fake = True
 		pixbuf = gtkgui_helpers.get_avatar_pixbuf_from_cache(jid_with_resource,
 			is_fake)
 		if pixbuf == 'ask':
 			# we don't have the vcard
-			gajim.connections[self.account].request_vcard(jid_with_resource,
-				is_fake)
+			if self.TYPE_ID == message_control.TYPE_PM:
+				if self.gc_contact.jid:
+					# We know the real jid of this contact
+					real_jid = self.gc_contact.jid
+					if self.gc_contact.resource:
+						real_jid += '/' + self.gc_contact.resource
+				else:
+					real_jid = jid_with_resource
+				gajim.connections[self.account].request_vcard(real_jid,
+					jid_with_resource)
+			else:
+				gajim.connections[self.account].request_vcard(jid_with_resource)
 			return
 		if pixbuf is not None:
 			scaled_pixbuf = gtkgui_helpers.get_scaled_pixbuf(pixbuf, 'chat')
@@ -1774,6 +1807,8 @@ class ChatControl(ChatControlBase):
 		# so this line adds that
 		window.set_events(gtk.gdk.POINTER_MOTION_MASK)
 		window.set_app_paintable(True)
+		if gtk.gtk_version >= (2, 10, 0) and gtk.pygtk_version >= (2, 10, 0):
+			window.set_type_hint(gtk.gdk.WINDOW_TYPE_HINT_TOOLTIP)
 		
 		window.realize()
 		window.window.set_back_pixmap(pixmap, False) # make it transparent
