@@ -269,10 +269,7 @@ class GnuPG:
     """
 
     def __init__(self):
-        if os.name == 'nt':    
-            self.call = 'gpg.exe'
-        else:    
-            self.call = 'gpg'
+        self.call = 'gpg'
         self.passphrase = None
         self.options = Options()
     
