@@ -206,6 +206,8 @@ def get_contact_dict_for_account(account):
 			contacts_dict['%s (%s)' % (name, contact1.jid)] = contact1
 			contacts_dict['%s (%s)' % (name, jid)] = contact
 		else:
+			if contact.name == gajim.get_nick_from_jid(jid):
+				del contacts_dict[jid]
 			contacts_dict[name] = contact
 	return contacts_dict
 
@@ -462,7 +464,7 @@ def play_sound(event):
 
 def play_sound_file(path_to_soundfile):
 	if path_to_soundfile == 'beep':
-		print '\a' # make a speaker beep
+		exec_command('beep')
 		return
 	if path_to_soundfile is None or not os.path.exists(path_to_soundfile):
 		return
@@ -859,6 +861,8 @@ def reduce_chars_newlines(text, max_chars = 0, max_lines = 0):
 			lines = map(lambda e: _cut_if_long(e), lines)
 	if lines:
 		reduced_text = reduce(lambda e, e1: e + '\n' + e1, lines)
+		if reduced_text != text:
+			reduced_text += '...'
 	else:
 		reduced_text = ''
 	return reduced_text
@@ -942,3 +946,15 @@ def get_accounts_info():
 		accounts.append({'name': account, 'status_line': single_line, 
 				'show': status, 'message': message})
 	return accounts
+
+def datetime_tuple(timestamp):
+	'''Converts timestamp using strptime and the format: %Y%m%dT%H:%M:%S
+	Because of various datetime formats are used the following exceptions
+	are handled:
+		- Optional milliseconds appened to the string are removed
+		- XEP-082 datetime strings have all '-' cahrs removed to meet
+		  the above format.'''
+	timestamp = timestamp.split('.')[0]
+	timestamp = timestamp.replace('-', '')
+	from time import strptime
+	return strptime(timestamp, '%Y%m%dT%H:%M:%S')
