@@ -8,7 +8,7 @@
 ##                         Vincent Hanquez <tab@snarc.org>
 ## Copyright (C) 2005 Yann Le Boulanger <asterix@lagaule.org>
 ##                    Vincent Hanquez <tab@snarc.org>
-##                    Nikos Kouremenos <nkour@jabber.org>
+##                    Nikos Kouremenos <kourem@gmail.com>
 ##                    Dimitur Kirov <dkirov@gmail.com>
 ##                    Travis Shirk <travis@pobox.com>
 ##                    Norman Rasmussen <norman@rasmussen.co.za>
@@ -36,21 +36,23 @@ else:
 # set '' so each part of the locale that should be modified is set
 # according to the environment variables
 locale.setlocale(locale.LC_ALL, '')
-_translation = None
 
-def init():
-	global _translation
-	try:
-		_translation = gettext.translation(APP, DIR)
-	except IOError:
-		_translation = gettext.NullTranslations()
+## For windows: set, if needed, a value in LANG environmental variable ##
+if os.name == 'nt':
+	lang = os.getenv('LANG')
+	if lang is None:
+		default_lang = locale.getdefaultlocale()[0] # en_US, fr_FR, el_GR etc..
+		if default_lang:
+			lang = default_lang
 
-init()
+	if lang:
+		os.environ['LANG'] = lang
 
-def _(s):
-	if s == '':
-		return s
-	return _translation.ugettext(s)
+gettext.install(APP, DIR, unicode = True)
+if gettext._translations:
+	_translation = gettext._translations.values()[0]
+else:
+	_translation = gettext.NullTranslations()
 
 def Q_(s):
 	# Qualified translatable strings
