@@ -5,12 +5,8 @@
  *
  * Copyright (C) 2003-2004 Yann Le Boulanger <asterix@lagaule.org>
  *                         Vincent Hanquez <tab@snarc.org>
- * Copyright (C) 2005 Yann Le Boulanger <asterix@lagaule.org>
- *                    Vincent Hanquez <tab@snarc.org>
+ * Copyright (C) 2005-2006 Yann Le Boulanger <asterix@lagaule.org>
  *                    Nikos Kouremenos <nkour@jabber.org>
- *                    Dimitur Kirov <dkirov@gmail.com>
- *                    Travis Shirk <travis@pobox.com>
- *                    Norman Rasmussen <norman@rasmussen.co.za>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published
@@ -26,19 +22,11 @@
 	#include <X11/Xlib.h>
 	#include <X11/Xutil.h>
 	#include <X11/extensions/scrnsaver.h>
-#else
-	#define _WIN32_WINNT 0x0500
-	#include <windows.h>
-	#define EXPORT __declspec(dllexport)
 #endif
 
 #include <Python.h>
 
-#ifdef _WIN32
-	typedef BOOL (WINAPI *GETLASTINPUTINFO)(LASTINPUTINFO *);
-	static HMODULE g_user32 = NULL;
-	static GETLASTINPUTINFO g_GetLastInputInfo = NULL;
-#else
+#ifndef _WIN32
 	Display *display;
 #endif
 
@@ -47,11 +35,6 @@ static PyObject * idle_init(PyObject *self, PyObject *args)
 {
 #ifndef _WIN32
 	display = XOpenDisplay(NULL);
-#else
-	g_user32 = LoadLibrary("user32.dll");
-	if (g_user32) {
-		g_GetLastInputInfo = (GETLASTINPUTINFO)GetProcAddress(g_user32, "GetLastInputInfo");
-	}
 #endif
 	Py_INCREF(Py_None);
 	return Py_None;
@@ -62,8 +45,6 @@ static PyObject * idle_getIdleSec(PyObject *self, PyObject *args)
 #ifndef _WIN32
 	static XScreenSaverInfo *mit_info = NULL;
 	int idle_time, event_base, error_base;
-#else
-	int idle_time = 0;
 #endif
 
 #ifndef _WIN32
@@ -76,16 +57,6 @@ static PyObject * idle_getIdleSec(PyObject *self, PyObject *args)
 	}
 	else
 		idle_time = 0;
-#else
-	if (g_GetLastInputInfo != NULL) {
-		LASTINPUTINFO lii;
-		memset(&lii, 0, sizeof(lii));
-		lii.cbSize = sizeof(lii);
-		if (g_GetLastInputInfo(&lii)) {
-			idle_time = lii.dwTime;
-		}
-		idle_time = (GetTickCount() - idle_time) / 1000;
-	}									
 #endif
 	return Py_BuildValue("i", idle_time);
 }
@@ -94,9 +65,6 @@ static PyObject * idle_close(PyObject *self, PyObject *args)
 {
 #ifndef _WIN32
 	XCloseDisplay(display);
-#else
-	if (g_user32 != NULL)
-		FreeLibrary(g_user32);
 #endif
 	Py_INCREF(Py_None);
 	return Py_None;
