@@ -57,7 +57,7 @@ class OptionsParser:
 			if os.path.exists(self.__filename):
 				#we talk about a file
 				print _('error: cannot open %s for reading') % self.__filename
-			return
+			return False
 
 		new_version = gajim.config.get('version')
 		for line in fd.readlines():
@@ -72,6 +72,7 @@ class OptionsParser:
 		self.old_values = {} # clean mem
 
 		fd.close()
+		return True
 
 	def write_line(self, fd, opt, parents, value):
 		if value == None:
@@ -151,6 +152,16 @@ class OptionsParser:
 			self.update_config_to_01018()
 		if old < [0, 11, 0, 1] and new >= [0, 11, 0, 1]:
 			self.update_config_to_01101()
+		if old < [0, 11, 0, 2] and new >= [0, 11, 0, 2]:
+			self.update_config_to_01102()
+		if old < [0, 11, 1, 1] and new >= [0, 11, 1, 1]:
+			self.update_config_to_01111()
+		if old < [0, 11, 1, 2] and new >= [0, 11, 1, 2]:
+			self.update_config_to_01112()
+		if old < [0, 11, 1, 3] and new >= [0, 11, 1, 3]:
+			self.update_config_to_01113()
+		if old < [0, 11, 1, 4] and new >= [0, 11, 1, 4]:
+			self.update_config_to_01114()
 
 		gajim.logger.init_vars()
 		gajim.config.set('version', new_version)
@@ -176,7 +187,7 @@ class OptionsParser:
 			'groupfontattrs', 'contacttextcolor', 'contactbgcolor', 'contactfont',
 			'contactfontattrs', 'bannertextcolor', 'bannerbgcolor', 'bannerfont', 
 			'bannerfontattrs']
-		for theme_name in (_('grocery'), _('gtk+')):
+		for theme_name in (_('grocery'), _('default')):
 			if theme_name not in gajim.config.get_per('themes'):
 				gajim.config.add_per('themes', theme_name)
 				theme = gajim.config.themes_default[theme_name]
@@ -207,7 +218,6 @@ class OptionsParser:
 
 	def assert_unread_msgs_table_exists(self):
 		'''create table unread_messages if there is no such table'''
-		#FIXME see #2812
 		back = os.getcwd()
 		os.chdir(logger.LOG_DB_FOLDER)
 		con = sqlite.connect(logger.LOG_DB_FILE)
@@ -375,3 +385,72 @@ class OptionsParser:
 			gajim.config.set('time_stamp', '%s%%X%s ' % (
 				self.old_values['before_time'], self.old_values['after_time']))
 		gajim.config.set('version', '0.11.0.1')
+
+	def update_config_to_01102(self):
+		'''fill time_stamp from before_time and after_time'''
+		if self.old_values.has_key('ft_override_host_to_send'):
+			gajim.config.set('ft_add_hosts_to_send',
+				self.old_values['ft_override_host_to_send'])
+		gajim.config.set('version', '0.11.0.2')
+	
+	def update_config_to_01111(self):
+		'''always_hide_chatbuttons -> compact_view'''
+		if self.old_values.has_key('always_hide_groupchat_buttons') and \
+		self.old_values.has_key('always_hide_chat_buttons'):
+			gajim.config.set('compact_view', self.old_values['always_hide_groupchat_buttons'] and \
+			self.old_values['always_hide_chat_buttons'])
+		gajim.config.set('version', '0.11.1.1')
+
+	def update_config_to_01112(self):
+		'''gtk+ theme is renamed to default'''
+		if self.old_values.has_key('roster_theme') and \
+		self.old_values['roster_theme'] == 'gtk+':
+			gajim.config.set('roster_theme', _('default'))
+		gajim.config.set('version', '0.11.1.2')
+
+	def update_config_to_01113(self):
+		# copy&pasted from update_config_to_01013
+		back = os.getcwd()
+		os.chdir(logger.LOG_DB_FOLDER)
+		con = sqlite.connect(logger.LOG_DB_FILE)
+		os.chdir(back)
+		cur = con.cursor()
+		try:
+			cur.executescript(
+				'''
+				CREATE TABLE caps_cache (
+					node TEXT,
+					ver TEXT,
+					ext TEXT,
+					data BLOB
+				);
+				'''
+			)
+			con.commit()
+		except sqlite.OperationalError, e:
+			pass
+		con.close()
+		gajim.config.set('version', '0.11.1.3')
+
+	def update_config_to_01114(self):
+		# add default theme if it doesn't exist
+		d = ['accounttextcolor', 'accountbgcolor', 'accountfont',
+			'accountfontattrs', 'grouptextcolor', 'groupbgcolor', 'groupfont',
+			'groupfontattrs', 'contacttextcolor', 'contactbgcolor', 'contactfont',
+			'contactfontattrs', 'bannertextcolor', 'bannerbgcolor', 'bannerfont', 
+			'bannerfontattrs']
+		theme_name = _('default')
+		if theme_name not in gajim.config.get_per('themes'):
+			gajim.config.add_per('themes', theme_name)
+			if gajim.config.get_per('themes', 'gtk+'):
+				# copy from old gtk+ theme
+				for o in d:
+					val = gajim.config.get_per('themes', 'gtk+', o)
+					gajim.config.set_per('themes', theme_name, o, val)
+				gajim.config.del_per('themes', 'gtk+')
+			else:
+				# copy from default theme
+				theme = gajim.config.themes_default[theme_name]
+				for o in d:
+					gajim.config.set_per('themes', theme_name, o, theme[d.index(o)])
+		gajim.config.set('version', '0.11.1.4')
