@@ -33,7 +33,7 @@ def ustr(what):
 	if type(r)<>type(u''): return unicode(r,ENCODING)
 	return r
 
-class Node:
+class Node(object):
 	""" Node class describes syntax of separate XML Node. It have a constructor that permits node creation
 		from set of "namespace name", attributes and payload of text strings and other nodes.
 		It does not natively support building node from text string and uses NodeBuilder class for that purpose.
@@ -58,9 +58,9 @@ class Node:
 			"node" and other arguments is provided then the node initially created as replica of "node"
 			provided and then modified to be compliant with other arguments."""
 		if node:
-			if self.FORCE_NODE_RECREATION and type(node)==type(self): 
+			if self.FORCE_NODE_RECREATION and isinstance(node, Node): 
 				node=str(node)
-			if type(node)<>type(self): 
+			if not isinstance(node, Node): 
 				node=NodeBuilder(node,self)
 			else:
 				self.name,self.namespace,self.attrs,self.data,self.kids,self.parent = node.name,node.namespace,{},[],[],node.parent
@@ -74,9 +74,9 @@ class Node:
 		if self.parent and not self.namespace: self.namespace=self.parent.namespace
 		for attr in attrs.keys():
 			self.attrs[attr]=attrs[attr]
-		if type(payload) in (type(''),type(u'')): payload=[payload]
+		if isinstance(payload, basestring): payload=[payload]
 		for i in payload:
-			if type(i)==type(self): self.addChild(node=i)
+			if isinstance(i, Node): self.addChild(node=i)
 			else: self.data.append(ustr(i))
 
 	def __str__(self,fancy=0):
@@ -130,7 +130,7 @@ class Node:
 	def delChild(self, node, attrs={}):
 		""" Deletes the "node" from the node's childs list, if "node" is an instance.
 			Else deletes the first node that have specified name and (optionally) attributes. """
-		if type(node)<>type(self): node=self.getTag(node,attrs)
+		if not isinstance(node, Node): node=self.getTag(node,attrs)
 		self.kids.remove(node)
 		return node
 	def getAttrs(self):
@@ -191,6 +191,18 @@ class Node:
 				else: nodes.append(node)
 			if one and nodes: return nodes[0]
 		if not one: return nodes
+	
+	def iterTags(self, name, attrs={}, namespace=None):
+		""" Iterate over all children using specified arguments as filter. """
+		for node in self.kids:
+			if namespace is not None and namespace!=node.getNamespace(): continue
+			if node.getName() == name:
+				for key in attrs.keys():
+					if not node.attrs.has_key(key) or \
+						node.attrs[key]!=attrs[key]: break
+				else:
+					yield node
+
 	def setAttr(self, key, val):
 		""" Sets attribute "key" with the value "val". """
 		self.attrs[key]=val
@@ -240,7 +252,7 @@ class Node:
 		return self.setAttr(item,val)
 	def __delitem__(self,item):
 		""" Deletes node's attribute "item". """
-		return self.delAttr(item,val)
+		return self.delAttr(item)
 	def __getattr__(self,attr):
 		""" Reduce memory usage caused by T/NT classes - use memory only when needed. """
 		if attr=='T':
@@ -290,6 +302,8 @@ class NodeBuilder:
 		self.Parse = self._parser.Parse
 
 		self.__depth = 0
+		self.__last_depth = 0
+		self.__max_depth = 0
 		self._dispatch_depth = 1
 		self._document_attrs = None
 		self._mini_dom=initial_node
@@ -326,7 +340,7 @@ class NodeBuilder:
 			ns=attr[:sp]           #
 			attrs[self.namespaces[ns]+attr[sp+1:]]=attrs[attr]
 			del attrs[attr]        #
-		self.__depth += 1
+		self._inc_depth()
 		self.DEBUG(DBG_NODEBUILDER, "DEPTH -> %i , tag -> %s, attrs -> %s" % (self.__depth, tag, `attrs`), 'down')
 		if self.__depth == self._dispatch_depth:
 			if not self._mini_dom : 
@@ -354,7 +368,7 @@ class NodeBuilder:
 			self._ptr = self._ptr.parent
 		else:
 			self.DEBUG(DBG_NODEBUILDER, "Got higher than dispatch level. Stream terminated?", 'stop')
-		self.__depth -= 1
+		self._dec_depth()
 		self.last_is_data = 0
 		if self.__depth == 0: self.stream_footer_received()
 	
@@ -362,7 +376,7 @@ class NodeBuilder:
 		if self.last_is_data:
 			if self.data_buffer:
 				self.data_buffer.append(data)
-		else:
+		elif self._ptr:
 			self.data_buffer = [data]
 			self.last_is_data = 1
 	
@@ -387,6 +401,19 @@ class NodeBuilder:
 		""" Method called when stream just closed. """
 		self.check_data_buffer()
 
+	def has_received_endtag(self, level=0):
+		""" Return True if at least one end tag was seen (at level) """
+		return self.__depth <= level and self.__max_depth > level
+
+	def _inc_depth(self):
+		self.__last_depth = self.__depth
+		self.__depth += 1
+		self.__max_depth = max(self.__depth, self.__max_depth)
+
+	def _dec_depth(self):
+		self.__last_depth = self.__depth
+		self.__depth -= 1
+
 def XML2Node(xml):
 	""" Converts supplied textual string into XML node. Handy f.e. for reading configuration file.
 		Raises xml.parser.expat.parsererror if provided string is not well-formed XML. """
