@@ -106,8 +106,17 @@ class NonBlockingTcp(PlugIn, IdleObject):
 			self._server = server
 		self.state = 0
 		try:
-			self._sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
-			self._sock.setblocking(False)
+			for ai in socket.getaddrinfo(server[0],server[1],socket.AF_UNSPEC,socket.SOCK_STREAM):
+				try:
+					self._sock=socket.socket(*ai[:3])
+					self._sock.setblocking(False)
+					self._server=ai[4]
+					break
+				except:
+					if sys.exc_value[0] == errno.EINPROGRESS:
+						break
+					#for all errors, we try other addresses
+					continue
 		except:
 			sys.exc_clear()
 			if self.on_connect_failure:
@@ -138,15 +147,16 @@ class NonBlockingTcp(PlugIn, IdleObject):
 			the owner's dispatcher. '''
 		self.disconnect()
 		self._owner.Connection = None
+		self._owner = None
 	
 	def pollin(self):
 		self._do_receive() 
 	
-	def pollend(self):
+	def pollend(self, retry = False):
+		conn_failure_cb = self.on_connect_failure
 		self.disconnect()
-		if self.on_connect_failure:
-			self.on_connect_failure()
-		self.on_connect_failure = None
+		if conn_failure_cb:
+			conn_failure_cb(retry)
 		
 	def disconnect(self):
 		if self.state == -2: # already disconnected
@@ -166,6 +176,7 @@ class NonBlockingTcp(PlugIn, IdleObject):
 		self.fd = -1
 		if self.on_disconnect:
 			self.on_disconnect()
+		self.on_connect_failure = None
 	
 	def end_disconnect(self):
 		''' force disconnect only if we are still trying to disconnect '''
@@ -214,20 +225,19 @@ class NonBlockingTcp(PlugIn, IdleObject):
 			# "received" will be empty anyhow 
 		if errnum == socket.SSL_ERROR_WANT_READ:
 			pass
-		elif errnum in [errno.ECONNRESET, errno.ENOTCONN, errno.ESHUTDOWN]:
-			self.disconnect()
-			if self.on_connect_failure:
-				self.on_connect_failure()
-			# don't proccess result, cas it will raise error
+		elif errnum  == errno.ECONNRESET:
+			self.pollend(True)
+			# don't proccess result, caus it will raise error
+			return
+		elif errnum in [errno.ENOTCONN, errno.ESHUTDOWN]:
+			self.pollend()
+			# don't proccess result, caus it will raise error
 			return
 		elif not received :
 			if errnum != socket.SSL_ERROR_EOF: 
 				# 8 EOF occurred in violation of protocol
 				self.DEBUG('Socket error while receiving data', 'error')
-				self.disconnect()
-				if self.on_connect_failure:
-					self.on_connect_failure(True)
-				self.on_connect_failure = None
+				self.pollend(True)
 			if self.state >= 0:
 				self.disconnect()
 			return
@@ -315,7 +325,7 @@ class NonBlockingTcp(PlugIn, IdleObject):
 			self.on_connect = None
 		return True
 
-	def send(self, raw_data):
+	def send(self, raw_data, now = False):
 		'''Append raw_data to the queue of messages to be send. 
 		If supplied data is unicode string, encode it to utf-8.
 		'''
@@ -326,7 +336,11 @@ class NonBlockingTcp(PlugIn, IdleObject):
 			r = r.encode('utf-8')
 		elif not isinstance(r, str): 
 			r = ustr(r).encode('utf-8')
-		self.sendqueue.append(r)
+		if now:
+			self.sendqueue.insert(0, r)
+			self._do_send()
+		else:
+			self.sendqueue.append(r)
 		self._plug_idle()
 
 	def _on_send(self):
@@ -374,8 +388,12 @@ class NonBlockingTLS(PlugIn):
 		PlugIn.PlugIn(self, owner)
 		DBG_LINE='NonBlockingTLS'
 		self.on_tls_start = on_tls_start
-		if now: 
-			res = self._startSSL()
+		if now:
+			try:
+				res = self._startSSL()
+			except Exception, e:
+				self._owner.socket.pollend()
+				return
 			self.tls_start()
 			return res
 		if self._owner.Dispatcher.Stream.features:
@@ -393,6 +411,8 @@ class NonBlockingTLS(PlugIn):
 		# if dispatcher is not plugged we cannot (un)register handlers
 		if self._owner.__dict__.has_key('Dispatcher'):
 			self._owner.UnregisterHandler('features', self.FeaturesHandler,xmlns=NS_STREAMS)
+			self._owner.Dispatcher.PlugOut()
+		self._owner = None
 
 	def tls_start(self):
 		if self.on_tls_start:
@@ -435,7 +455,11 @@ class NonBlockingTLS(PlugIn):
 			self.DEBUG('Got starttls response: ' + self.starttls,'error')
 			return
 		self.DEBUG('Got starttls proceed response. Switching to TLS/SSL...','ok')
-		self._startSSL()
+		try:
+			self._startSSL()
+		except Exception, e:
+			self._owner.socket.pollend()
+			return
 		self._owner.Dispatcher.PlugOut()
 		dispatcher_nb.Dispatcher().PlugIn(self._owner)
 
