@@ -17,12 +17,40 @@
 ## GNU General Public License for more details.
 ##
 
-import sys
 import os
+
+if os.name == 'nt':
+	import warnings
+	warnings.filterwarnings(action='ignore')
+
+# Used to create windows installer with GTK included
+#	paths = os.environ['PATH']
+#	list_ = paths.split(';')
+#	new_list = []
+#	for p in list_:
+#		if p.find('gtk') < 0 and p.find('GTK') < 0:
+#			new_list.append(p)
+#	new_list.insert(0, 'gtk/lib')
+#	new_list.insert(0, 'gtk/bin')
+#	os.environ['PATH'] = ';'.join(new_list)
+#	os.environ['GTK_BASEPATH'] = 'gtk'
+
+import sys
 import urllib
 
 from common import i18n
 
+# PyGTK2.10+ only throws a warning
+import warnings
+warnings.filterwarnings('error', module='gtk')
+try:
+	import gtk
+except Warning, msg:
+	if str(msg) == 'could not open display':
+		print >> sys.stderr, _('Gajim needs X server to run. Quiting...')
+		sys.exit()
+warnings.resetwarnings()
+
 import message_control
 
 from chat_control import ChatControlBase
@@ -40,12 +68,6 @@ if os.name == 'posix': # dl module is Unix Only
 	except:
 		pass
 
-try:
-	import gtk
-except RuntimeError, msg:
-	if str(msg) == 'could not open display':
-		print >> sys.stderr, _('Gajim needs X server to run. Quiting...')
-		sys.exit()
 pritext = ''
 if gtk.pygtk_version < (2, 6, 0):
 	pritext = _('Gajim needs PyGTK 2.6 or above')
@@ -608,7 +630,7 @@ class Interface:
 			# remove in 2007
 			# It's maybe a GC_NOTIFY (specialy for MSN gc)
 			self.handle_event_gc_notify(account, (jid, array[1], status_message,
-				array[3], None, None, None, None, None, None, None))
+				array[3], None, None, None, None, None, None, None, None))
 			
 
 	def handle_event_msg(self, account, array):
@@ -1020,18 +1042,32 @@ class Interface:
 		# print status in chat window and update status/GPG image
 		if self.msg_win_mgr.has_window(fjid, account):
 			ctrl = self.msg_win_mgr.get_control(fjid, account)
-			contact = ctrl.contact
-			contact.show = show
-			contact.status = status
-			ctrl.update_ui()
-			uf_show = helpers.get_uf_show(show)
-			if status:
-				ctrl.print_conversation(_('%s is now %s (%s)') % (nick, uf_show,
-					status), 'status')
+			statusCode = array[9]
+			if '303' in statusCode:
+				new_nick = array[10]
+				ctrl.print_conversation(_('%s is now known as %s') % (nick,
+					new_nick), 'status')
+				gc_c = gajim.contacts.get_gc_contact(account, room_jid, new_nick)
+				c = gajim.contacts.contact_from_gc_contact(gc_c)
+				ctrl.gc_contact = gc_c
+				ctrl.contact = c
+				ctrl.draw_banner()
+				old_jid = room_jid + '/' + nick
+				new_jid = room_jid + '/' + new_nick
+				self.msg_win_mgr.change_key(old_jid, new_jid, account)
 			else:
-				ctrl.print_conversation(_('%s is now %s') % (nick, uf_show),
-					'status')
-			ctrl.parent_win.redraw_tab(ctrl)
+				contact = ctrl.contact
+				contact.show = show
+				contact.status = status
+				uf_show = helpers.get_uf_show(show)
+				if status:
+					ctrl.print_conversation(_('%s is now %s (%s)') % (nick, uf_show,
+						status), 'status')
+				else:
+					ctrl.print_conversation(_('%s is now %s') % (nick, uf_show),
+						'status')
+				ctrl.parent_win.redraw_tab(ctrl)
+				ctrl.update_ui()
 			if self.remote_ctrl:
 				self.remote_ctrl.raise_signal('GCPresence', (account, array))
 
@@ -1772,7 +1808,8 @@ class Interface:
 		# because emoticons match later (in the string) they need to be after
 		# basic matches that may occur earlier
 		emot_and_basic_pattern = basic_pattern + emoticons_pattern
-		self.emot_and_basic_re = re.compile(emot_and_basic_pattern, re.IGNORECASE)
+		self.emot_and_basic_re = re.compile(emot_and_basic_pattern,
+			re.IGNORECASE + re.UNICODE)
 		
 		# at least one character in 3 parts (before @, after @, after .)
 		self.sth_at_sth_dot_sth_re = re.compile(r'\S+@\S+\.\S*[^\s)?]')
@@ -2040,7 +2077,7 @@ class Interface:
 			'urlmsgcolor': gajim.config.get('urlmsgcolor'),
 		}
 
-		parser.read()
+		cfg_was_read = parser.read()
 		# Do not set gajim.verbose to False if -v option was given
 		if gajim.config.get('verbose'):
 			gajim.verbose = True
@@ -2074,8 +2111,7 @@ class Interface:
 					gajim.config.set_per('themes', theme_name, o,
 						theme[d.index(o)])
 			
-		if gajim.config.get('autodetect_browser_mailer') or \
-		len(gajim.connections) == 0:
+		if gajim.config.get('autodetect_browser_mailer') or not cfg_was_read:
 			gtkgui_helpers.autodetect_browser_mailer()
 
 		if gajim.verbose:
