@@ -9,7 +9,7 @@
 ##                         Vincent Hanquez <tab@snarc.org>
 ## Copyright (C) 2005 Yann Le Boulanger <asterix@lagaule.org>
 ##                    Vincent Hanquez <tab@snarc.org>
-##                    Nikos Kouremenos <nkour@jabber.org>
+##                    Nikos Kouremenos <kourem@gmail.com>
 ##                    Dimitur Kirov <dkirov@gmail.com>
 ##                    Travis Shirk <travis@pobox.com>
 ##                    Norman Rasmussen <norman@rasmussen.co.za>
@@ -25,23 +25,18 @@
 ##
 
 import gtk
-import gtk.glade
 import pango
 import dialogs
 import gtkgui_helpers
 
 from common import gajim
-from common import i18n
-_ = i18n._
-APP = i18n.APP
-gtk.glade.bindtextdomain (APP, i18n.DIR)
-gtk.glade.textdomain (APP)
 
 class GajimThemesWindow:
 
 	def __init__(self):
 		self.xml = gtkgui_helpers.get_glade('gajim_themes_window.glade')
 		self.window = self.xml.get_widget('gajim_themes_window')
+		self.window.set_transient_for(gajim.interface.roster.window)
 		
 		self.options = ['account', 'group', 'contact', 'banner']
 		self.options_combobox = self.xml.get_widget('options_combobox')
@@ -56,7 +51,7 @@ class GajimThemesWindow:
 		self.themes_tree = self.xml.get_widget('themes_treeview')
 		self.theme_options_vbox = self.xml.get_widget('theme_options_vbox')
 		self.colorbuttons = {}
-		for chatstate in ('active', 'inactive', 'composing', 'paused', 'gone',
+		for chatstate in ('inactive', 'composing', 'paused', 'gone',
 		'muc_msg', 'muc_directed_msg'):
 			self.colorbuttons[chatstate] = self.xml.get_widget(chatstate + \
 				'_colorbutton')
@@ -86,6 +81,8 @@ class GajimThemesWindow:
 		return True # do NOT destroy the window
 	
 	def on_close_button_clicked(self, widget):
+		if gajim.interface.instances.has_key('preferences'):
+			gajim.interface.instances['preferences'].update_theme_list()
 		self.window.hide()
 
 	def on_theme_cell_edited(self, cell, row, new_name):
@@ -126,7 +123,20 @@ class GajimThemesWindow:
 				self.themes_tree.get_selection().select_iter(iter)
 				self.xml.get_widget('remove_button').set_sensitive(True)
 				self.theme_options_vbox.set_sensitive(True)
-	
+
+	def select_active_theme(self):
+		model = self.themes_tree.get_model()
+		iter = model.get_iter_root()
+		active_theme = gajim.config.get('roster_theme')
+		while iter:
+			theme = model[iter][0]
+			if theme == active_theme:
+				self.themes_tree.get_selection().select_iter(iter)
+				self.xml.get_widget('remove_button').set_sensitive(True)
+				self.theme_options_vbox.set_sensitive(True)
+				break
+			iter = model.iter_next(iter)
+
 	def selection_changed(self, widget = None):
 		(model, iter) = self.themes_tree.get_selection().get_selected()
 		selected = self.themes_tree.get_selection().get_selected_rows()
@@ -203,7 +213,7 @@ class GajimThemesWindow:
 		self.no_update = False
 		gajim.interface.roster.change_roster_style(None)
 
-		for chatstate in ('active', 'inactive', 'composing', 'paused', 'gone',
+		for chatstate in ('inactive', 'composing', 'paused', 'gone',
 		'muc_msg', 'muc_directed_msg'):
 			color = gajim.config.get_per('themes', theme, 'state_' + chatstate + \
 				'_color')
@@ -338,11 +348,6 @@ class GajimThemesWindow:
 			font_props[1] = True
 		return font_props
 
-	def on_active_colorbutton_color_set(self, widget):
-		self.no_update = True
-		self._set_color(True, widget, 'state_active_color')
-		self.no_update = False
-
 	def on_inactive_colorbutton_color_set(self, widget):
 		self.no_update = True
 		self._set_color(True, widget, 'state_inactive_color')
