@@ -63,6 +63,12 @@ def tree_cell_data_func(column, renderer, model, iter, tv=None):
 	# cell data func is global, because we don't want it to keep
 	# reference to GroupchatControl instance (self)
 	theme = gajim.config.get('roster_theme')
+	# allocate space for avatar only if needed
+	if isinstance(renderer, gtk.CellRendererPixbuf):
+		if model[iter][C_AVATAR]:
+			renderer.set_property('visible', True)
+		else:
+			renderer.set_property('visible', False)
 	if model.iter_parent(iter):
 		bgcolor = gajim.config.get_per('themes', theme, 'contactbgcolor')
 		if bgcolor:
@@ -97,10 +103,11 @@ def tree_cell_data_func(column, renderer, model, iter, tv=None):
 class PrivateChatControl(ChatControl):
 	TYPE_ID = message_control.TYPE_PM
 
-	def __init__(self, parent_win, contact, acct):
+	def __init__(self, parent_win, gc_contact, contact, acct):
 		room_jid = contact.jid.split('/')[0]
 		room_ctrl = gajim.interface.msg_win_mgr.get_control(room_jid, acct)
 		self.room_name = room_ctrl.name
+		self.gc_contact = gc_contact
 		ChatControl.__init__(self, parent_win, contact, acct)
 		self.TYPE_ID = 'pm'
 
@@ -179,7 +186,8 @@ class GroupchatControl(ChatControlBase):
 		self.handlers[id] = widget
 
 		self.room_jid = self.contact.jid
-		self.nick = contact.name
+		self.nick = contact.name.decode('utf-8')
+		self.new_nick = ''
 		self.name = self.room_jid.split('@')[0]
 
 		hide_chat_buttons_always = gajim.config.get(
@@ -208,10 +216,10 @@ class GroupchatControl(ChatControlBase):
 		# connect the menuitems to their respective functions
 		xm = gtkgui_helpers.get_glade('gc_control_popup_menu.glade')
 
-		widget = xm.get_widget('bookmark_room_menuitem')
-		id = widget.connect('activate',
+		self.bookmark_room_menuitem = xm.get_widget('bookmark_room_menuitem')
+		id = self.bookmark_room_menuitem.connect('activate',
 			self._on_bookmark_room_menuitem_activate)
-		self.handlers[id] = widget
+		self.handlers[id] = self.bookmark_room_menuitem
 
 		widget = xm.get_widget('change_nick_menuitem')
 		id = widget.connect('activate', self._on_change_nick_menuitem_activate)
@@ -270,14 +278,8 @@ class GroupchatControl(ChatControlBase):
 		# first one img, second one text, third is sec pixbuf
 		column = gtk.TreeViewColumn()
 
-		renderer_pixbuf = gtk.CellRendererPixbuf() # avatar image
-		column.pack_start(renderer_pixbuf, expand = False)
-		column.add_attribute(renderer_pixbuf, 'pixbuf', C_AVATAR)
-		column.set_cell_data_func(renderer_pixbuf, tree_cell_data_func,
-			self.list_treeview)
-		renderer_pixbuf.set_property('xalign', 1) # align pixbuf to the right
-
 		renderer_image = cell_renderer_image.CellRendererImage(0, 0) # status img
+		renderer_image.set_property('width', 26)
 		column.pack_start(renderer_image, expand = False)
 		column.add_attribute(renderer_image, 'image', C_IMG)
 		column.set_cell_data_func(renderer_image, tree_cell_data_func, 
@@ -286,9 +288,17 @@ class GroupchatControl(ChatControlBase):
 		renderer_text = gtk.CellRendererText() # nickname
 		column.pack_start(renderer_text, expand = True)
 		column.add_attribute(renderer_text, 'markup', C_TEXT)
+		renderer_text.set_property("ellipsize", pango.ELLIPSIZE_END)
 		column.set_cell_data_func(renderer_text, tree_cell_data_func,
 			self.list_treeview)
 
+		renderer_pixbuf = gtk.CellRendererPixbuf() # avatar image
+		column.pack_start(renderer_pixbuf, expand = False)
+		column.add_attribute(renderer_pixbuf, 'pixbuf', C_AVATAR)
+		column.set_cell_data_func(renderer_pixbuf, tree_cell_data_func,
+			self.list_treeview)
+		renderer_pixbuf.set_property('xalign', 1) # align pixbuf to the right
+
 		self.list_treeview.append_column(column)
 
 		# workaround to avoid gtk arrows to be shown
@@ -474,6 +484,8 @@ class GroupchatControl(ChatControlBase):
 		childs = menu.get_children()
 		# hide chat buttons
 		childs[5].set_active(self.hide_chat_buttons_current)
+		if not gajim.connections[self.account].private_storage_supported:
+			self.bookmark_room_menuitem.set_sensitive(False)
 		if gajim.gc_connected[self.account][self.room_jid]:
 			c = gajim.contacts.get_gc_contact(self.account, self.room_jid,
 				self.nick)
@@ -497,7 +509,7 @@ class GroupchatControl(ChatControlBase):
 			else:
 				self.print_conversation(msg, nick, tim, xhtml)
 
-	def on_private_message(self, nick, msg, tim, xhtml):
+	def on_private_message(self, nick, msg, tim, xhtml, msg_id = None):
 		# Do we have a queue?
 		fjid = self.room_jid + '/' + nick
 		no_queue = len(gajim.events.get_events(self.account, fjid)) == 0
@@ -509,7 +521,7 @@ class GroupchatControl(ChatControlBase):
 			return
 
 		event = gajim.events.create_event('pm', (msg, '', 'incoming', tim,
-			False, '', None, xhtml))
+			False, '', msg_id, xhtml))
 		gajim.events.add_event(self.account, fjid, event)
 
 		autopopup = gajim.config.get('autopopup')
@@ -758,6 +770,8 @@ class GroupchatControl(ChatControlBase):
 			self.add_contact_to_roster(nick, gc_contact.show, gc_contact.role,
 						gc_contact.affiliation, gc_contact.status,
 						gc_contact.jid)
+		# Recalculate column width for ellipsizin
+		self.list_treeview.columns_autosize()
 
 	def on_send_pm(self, widget = None, model = None, iter = None, nick = None,
 	msg = None):
@@ -819,7 +833,7 @@ class GroupchatControl(ChatControlBase):
 		model[iter][C_AVATAR] = scaled_pixbuf
 
 	def chg_contact_status(self, nick, show, status, role, affiliation, jid,
-	reason, actor, statusCode, new_nick):
+	reason, actor, statusCode, new_nick, avatar_sha, tim = None):
 		'''When an occupant changes his or her status'''
 		if show == 'invisible':
 			return
@@ -828,10 +842,10 @@ class GroupchatControl(ChatControlBase):
 			role = 'visitor'
 		if not affiliation:
 			affiliation = 'none'
-
+		fake_jid = self.room_jid + '/' + nick
 		newly_created = False
 		if show in ('offline', 'error'):
-			if statusCode == '307':
+			if '307' in statusCode:
 				if actor is None: # do not print 'kicked by None'
 					s = _('%(nick)s has been kicked: %(reason)s') % {
 						'nick': nick,
@@ -842,7 +856,7 @@ class GroupchatControl(ChatControlBase):
 						'who': actor,
 						'reason': reason }
 				self.print_conversation(s, 'info')
-			elif statusCode == '301':
+			elif '301' in statusCode:
 				if actor is None: # do not print 'banned by None'
 					s = _('%(nick)s has been banned: %(reason)s') % {
 						'nick': nick,
@@ -853,9 +867,10 @@ class GroupchatControl(ChatControlBase):
 						'who': actor,
 						'reason': reason }
 				self.print_conversation(s, 'info')
-			elif statusCode == '303': # Someone changed his or her nick
-				if nick == self.nick: # We changed our nick
-					self.nick = new_nick
+			elif '303' in statusCode: # Someone changed his or her nick
+				if new_nick == self.new_nick: # We changed our nick
+					self.nick = self.new_nick
+					self.new_nick = ''
 					s = _('You are now known as %s') % new_nick
 				else:
 					s = _('%s is now known as %s') % (nick, new_nick)
@@ -882,21 +897,20 @@ class GroupchatControl(ChatControlBase):
 					files[os.path.join(path, puny_nick + ext)] = \
 						os.path.join(path, puny_new_nick + ext)
 				for old_file in files:
-					if os.path.exists(old_file):
+					if os.path.exists(old_file) and old_file != files[old_file]:
 						if os.path.exists(files[old_file]):
 							# Windows require this
 							os.remove(files[old_file])
 						os.rename(old_file, files[old_file])
 				self.print_conversation(s, 'info')
 
-			if len(gajim.events.get_events(self.account,
-			self.room_jid + '/' + nick)) == 0:
+			if len(gajim.events.get_events(self.account, fake_jid)) == 0:
 				self.remove_contact(nick)
 			else:
 				c = gajim.contacts.get_gc_contact(self.account, self.room_jid, nick)
 				c.show = show
 				c.status = status
-			if nick == self.nick and statusCode != '303': # We became offline
+			if nick == self.nick and '303' not in statusCode: # We became offline
 				self.got_disconnected()
 				self.parent_win.redraw_tab(self)
 		else:
@@ -905,28 +919,60 @@ class GroupchatControl(ChatControlBase):
 				iter = self.add_contact_to_roster(nick, show, role, affiliation,
 					status, jid)
 				newly_created = True
-				if statusCode == '201': # We just created the room
+				if '201' in statusCode: # We just created the room
 					gajim.connections[self.account].request_gc_config(self.room_jid)
 			else:
+				gc_c = gajim.contacts.get_gc_contact(self.account, self.room_jid,
+					nick)
+				# Re-get vcard if avatar has changed
+				# We do that here because we may request it to the real JID if we
+				# knows it. connections.py doesn't know it.
+				con = gajim.connections[self.account]
+				if gc_c.jid:
+					real_jid = gc_c.jid
+					if gc_c.resource:
+						real_jid += '/' + gc_c.resource
+				else:
+					real_jid = fake_jid
+				if con.vcard_shas.has_key(fake_jid):
+					if avatar_sha != con.vcard_shas[fake_jid]:
+						server = gajim.get_server_from_jid(self.room_jid)
+						if not server.startswith('irc'):
+							con.request_vcard(real_jid, fake_jid)
+				else:
+					cached_vcard = con.get_cached_vcard(fake_jid, True)
+					if cached_vcard and cached_vcard.has_key('PHOTO') and \
+					cached_vcard['PHOTO'].has_key('SHA'):
+						cached_sha = cached_vcard['PHOTO']['SHA']
+					else:
+						cached_sha = ''
+					if cached_sha != avatar_sha:
+						# avatar has been updated
+						# sha in mem will be updated later
+						server = gajim.get_server_from_jid(self.room_jid)
+						if not server.startswith('irc'):
+							con.request_vcard(real_jid, fake_jid)
+					else:
+						# save sha in mem NOW
+						con.vcard_shas[fake_jid] = avatar_sha
+
 				actual_role = self.get_role(nick)
 				if role != actual_role:
 					self.remove_contact(nick)
 					self.add_contact_to_roster(nick, show, role,
 						affiliation, status, jid)
 				else:
-					c = gajim.contacts.get_gc_contact(self.account, self.room_jid,
-						nick)
-					if c.show == show and c.status == status and \
-						c.affiliation == affiliation: #no change
+					if gc_c.show == show and gc_c.status == status and \
+						gc_c.affiliation == affiliation: # no change
 						return
-					c.show = show
-					c.affiliation = affiliation
-					c.status = status
+					gc_c.show = show
+					gc_c.affiliation = affiliation
+					gc_c.status = status
 					self.draw_contact(nick)
 
 		self.parent_win.redraw_tab(self)
 		if (time.time() - self.room_creation) > 30 and \
-				nick != self.nick and statusCode != '303':
+				nick != self.nick and '303' not in statusCode:
 			st = ''
 			print_status = None
 			for bookmark in gajim.connections[self.account].bookmarks:
@@ -940,7 +986,8 @@ class GroupchatControl(ChatControlBase):
 				# delete ressource
 				simple_jid = gajim.get_jid_without_resource(jid)
 				nick_jid += ' (%s)' % simple_jid
-			if show == 'offline' and print_status in ('all', 'in_and_out'):
+			if show == 'offline' and print_status in ('all', 'in_and_out') and \
+			'307' not in statusCode:
 				st = _('%s has left') % nick_jid
 				if reason:
 					st += ' [%s]' % reason
@@ -987,10 +1034,16 @@ class GroupchatControl(ChatControlBase):
 		server = gajim.get_server_from_jid(self.room_jid)
 		if gajim.config.get('ask_avatars_on_startup') and \
 		not server.startswith('irc'):
-			fjid = self.room_jid + '/' + nick
-			pixbuf = gtkgui_helpers.get_avatar_pixbuf_from_cache(fjid, True)
+			fake_jid = self.room_jid + '/' + nick
+			pixbuf = gtkgui_helpers.get_avatar_pixbuf_from_cache(fake_jid, True)
 			if pixbuf == 'ask':
-				gajim.connections[self.account].request_vcard(fjid, True)
+				if j:
+					fjid = j
+					if resource:
+						fjid += '/' + resource
+					gajim.connections[self.account].request_vcard(fjid, fake_jid)
+				else:
+					gajim.connections[self.account].request_vcard(fake_jid, fake_jid)
 		if nick == self.nick: # we became online
 			self.got_connected()
 		self.list_treeview.expand_row((model.get_path(role_iter)), False)
@@ -1047,8 +1100,15 @@ class GroupchatControl(ChatControlBase):
 			# example: /nick foo
 			if len(message_array) and message_array[0] != self.nick:
 				nick = message_array[0]
-				nick = helpers.parse_resource(nick)
+				try:
+					nick = helpers.parse_resource(nick)
+				except:
+					# Invalid Nickname
+					dialogs.ErrorDialog(_('Invalid nickname'),
+					_('The nickname has not allowed characters.'))
+					return True
 				gajim.connections[self.account].join_gc(nick, self.room_jid, None)
+				self.new_nick = nick
 				self.clear(self.msg_textview)
 			else:
 				self.get_command_help(command)
@@ -1111,19 +1171,13 @@ class GroupchatControl(ChatControlBase):
 			if len(message_array):
 				message_array = message_array[0].split()
 				invitee = message_array.pop(0)
-				if invitee.find('@') >= 0:
-					reason = ' '.join(message_array)
-					gajim.connections[self.account].send_invite(self.room_jid,
-						invitee, reason)
-					s = _('Invited %(contact_jid)s to %(room_jid)s.') % {
-						'contact_jid': invitee,
-						'room_jid': self.room_jid}
-					self.print_conversation(s, 'info')
-					self.clear(self.msg_textview)
-				else:
-					#%s is something the user wrote but it is not a jid so we inform
-					s = _('%s does not appear to be a valid JID') % invitee
-					self.print_conversation(s, 'info')
+				reason = ' '.join(message_array)
+				gajim.connections[self.account].send_invite(self.room_jid, invitee, reason)
+				s = _('Invited %(contact_jid)s to %(room_jid)s.') % {
+					'contact_jid': invitee,
+					'room_jid': self.room_jid}
+				self.print_conversation(s, 'info')
+				self.clear(self.msg_textview)
 			else:
 				self.get_command_help(command)
 			return True
@@ -1328,11 +1382,27 @@ class GroupchatControl(ChatControlBase):
 		'''asks user for new nick and on ok it sets it on room'''
 		def on_ok(widget):
 			nick = instance.input_entry.get_text().decode('utf-8')
-			nick = helpers.parse_resource(nick)
+			try:
+				nick = helpers.parse_resource(nick)
+			except:
+				# invalid char
+				dialogs.ErrorDialog(_('Invalid nickname'),
+				_('The nickname has not allowed characters.'))
+				return
 			gajim.connections[self.account].join_gc(nick, self.room_jid, None)
-			self.nick = nick
+			if gajim.gc_connected[self.account][self.room_jid]:
+				# We are changing nick, we will change self.nick when we receive 
+				# presence that inform that it works
+				self.new_nick = nick
+			else:
+				# We are connecting, we will not get a changed nick presence so
+				# change it NOW. We don't already have a nick so it's harmless
+				self.nick = nick
+		def on_cancel():
+			self.new_nick = '' 
+
 		instance = dialogs.InputDialog(title, prompt, proposed_nick,
-			is_modal = False, ok_handler = on_ok)
+			is_modal = False, ok_handler = on_ok, cancel_handler = on_cancel)
 
 	def shutdown(self, status='offline'):
 		gajim.connections[self.account].send_gc_status(self.nick, self.room_jid,
@@ -1707,12 +1777,11 @@ class GroupchatControl(ChatControlBase):
 
 	def _start_private_message(self, nick):
 		gc_c = gajim.contacts.get_gc_contact(self.account, self.room_jid, nick)
-		c = gajim.contacts.contact_from_gc_contact(gc_c)
-		nick_jid = c.jid
+		nick_jid = gc_c.get_full_jid()
 
 		win = gajim.interface.msg_win_mgr.get_window(nick_jid, self.account)
 		if not win:
-			gajim.interface.roster.new_chat(c, self.account, private_chat = True)
+			gajim.interface.roster.new_private_chat(gc_c, self.account)
 			win = gajim.interface.msg_win_mgr.get_window(nick_jid, self.account)
 		win.set_active_tab(nick_jid, self.account)
 		win.window.present()
