@@ -1,18 +1,7 @@
 ##	gtkexcepthook.py
 ##
-## Contributors for this file:
-##	- Yann Le Boulanger <asterix@lagaule.org>
-##	- Nikos Kouremenos <kourem@gmail.com>
-##	- Dimitur Kirov <dkirov@gmail.com>
-##
-## Copyright (C) 2003-2004 Yann Le Boulanger <asterix@lagaule.org>
-##                         Vincent Hanquez <tab@snarc.org>
-## Copyright (C) 2005 Yann Le Boulanger <asterix@lagaule.org>
-##                    Vincent Hanquez <tab@snarc.org>
-##                    Nikos Kouremenos <nkour@jabber.org>
-##                    Dimitur Kirov <dkirov@gmail.com>
-##                    Travis Shirk <travis@pobox.com>
-##                    Norman Rasmussen <norman@rasmussen.co.za>
+## Copyright (C) 2005-2006 Yann Le Boulanger <asterix@lagaule.org>
+## Copyright (C) 2005-2006 Nikos Kouremenos <kourem@gmail.com>
 ##
 ## Initially written and submitted by Gustavo J. A. M. Carneiro
 ##
@@ -27,18 +16,18 @@
 ##
 
 import sys
+import os
 import traceback
 import threading
 
 import gtk
 import pango
+from common import i18n
 import dialogs
 
 from cStringIO import StringIO
 from common import helpers
-from common import i18n
 
-_ = i18n._
 _exception_in_progress = threading.Lock()
 
 def _info(type, value, tb):
@@ -102,7 +91,8 @@ def _info(type, value, tb):
 
 	_exception_in_progress.release()
 	
-if not sys.stderr.isatty(): # gdb/kdm etc if we use startx this is not True
+# gdb/kdm etc if we use startx this is not True
+if os.name == 'nt' or not sys.stderr.isatty():
 	#FIXME: maybe always show dialog?
 	_excepthook_save = sys.excepthook
 	sys.excepthook = _info
